@@ -2,24 +2,34 @@
 
 static bool IsUnicode(byte *Data,int Size);
 
-bool ReadTextFile(char *Name,StringList *List,bool Config,bool AbortOnError,
-                  RAR_CHARSET SrcCharset,bool Unquote,bool SkipComments)
+bool ReadTextFile(
+  const wchar *Name,
+  StringList *List,
+  bool Config,
+  bool AbortOnError,
+  RAR_CHARSET SrcCharset,
+  bool Unquote,
+  bool SkipComments,
+  bool ExpandEnvStr)
 {
-  char FileName[NM];
-  if (Config)
-    GetConfigName(Name,FileName,true);
-  else
-    strcpy(FileName,Name);
+  wchar FileName[NM];
+  *FileName=0;
+
+  if (Name!=NULL)
+    if (Config)
+      GetConfigName(Name,FileName,ASIZE(FileName),true);
+    else
+      wcsncpyz(FileName,Name,ASIZE(FileName));
 
   File SrcFile;
-  if (*FileName)
+  if (FileName!=NULL && *FileName!=0)
   {
-    bool OpenCode=AbortOnError ? SrcFile.WOpen(FileName):SrcFile.Open(FileName);
+    bool OpenCode=AbortOnError ? SrcFile.WOpen(FileName):SrcFile.Open(FileName,0);
 
     if (!OpenCode)
     {
       if (AbortOnError)
-        ErrHandler.Exit(OPEN_ERROR);
+        ErrHandler.Exit(RARX_OPEN);
       return(false);
     }
   }
@@ -37,16 +47,21 @@ bool ReadTextFile(char *Name,StringList *List,bool Config,bool AbortOnError,
 
   memset(&Data[DataSize],0,5);
 
+  Array<wchar> WideStr;
+
   if (SrcCharset==RCH_UNICODE ||
       SrcCharset==RCH_DEFAULT && IsUnicode((byte *)&Data[0],DataSize))
   {
-    // Unicode in native system format, can be more than 2 bytes per character
+    // Unicode in native system format, can be more than 2 bytes per character.
     Array<wchar> DataW(Data.Size()/2+1);
-    for (int I=2;I<Data.Size()-1;I+=2)
-      DataW[(I-2)/2]=(wchar)Data[I]+(wchar)Data[I+1]*256;
+    for (size_t I=2;I<Data.Size()-1;I+=2)
+    {
+      // Need to convert Data to (byte) first to prevent the sign extension
+      // to higher bytes.
+      DataW[(I-2)/2]=(wchar)((byte)Data[I])+(wchar)((byte)Data[I+1])*256;
+    }
 
     wchar *CurStr=&DataW[0];
-    Array<char> AnsiName;
 
     while (*CurStr!=0)
     {
@@ -67,19 +82,32 @@ bool ReadTextFile(char *Name,StringList *List,bool Config,bool AbortOnError,
           break;
         *SpacePtr=0;
       }
-      if (*CurStr)
+      if (*CurStr!=0)
       {
-        int Length=strlenw(CurStr);
-        int AddSize=4*(Length-AnsiName.Size()+1);
-        if (AddSize>0)
-          AnsiName.Add(AddSize);
+        size_t Length=wcslen(CurStr);
+
         if (Unquote && *CurStr=='\"' && CurStr[Length-1]=='\"')
         {
           CurStr[Length-1]=0;
           CurStr++;
         }
-        WideToChar(CurStr,&AnsiName[0],AnsiName.Size());
-        List->AddString(&AnsiName[0],CurStr);
+
+        bool Expanded=false;
+#if defined(_WIN_ALL) && !defined(_WIN_CE)
+        if (ExpandEnvStr && *CurStr=='%')
+        {
+          // Expanding environment variables in Windows version.
+
+          wchar ExpName[NM];
+          *ExpName=0;
+          DWORD Result=ExpandEnvironmentStrings(CurStr,ExpName,ASIZE(ExpName));
+          Expanded=Result!=0 && Result<ASIZE(ExpName);
+          if (Expanded)
+            List->AddString(ExpName);
+        }
+#endif
+        if (!Expanded)
+          List->AddString(CurStr);
       }
       CurStr=NextStr+1;
       while (*CurStr=='\r' || *CurStr=='\n')
@@ -112,34 +140,51 @@ bool ReadTextFile(char *Name,StringList *List,bool Config,bool AbortOnError,
       {
         if (Unquote && *CurStr=='\"')
         {
-          int Length=strlen(CurStr);
+          size_t Length=strlen(CurStr);
           if (CurStr[Length-1]=='\"')
           {
             CurStr[Length-1]=0;
             CurStr++;
           }
         }
-#if defined(_WIN_32)
+#if defined(_WIN_ALL)
         if (SrcCharset==RCH_OEM)
-          OemToChar(CurStr,CurStr);
+          OemToCharA(CurStr,CurStr);
 #endif
-        List->AddString(CurStr);
+
+        bool Expanded=false;
+
+        WideStr.Alloc(strlen(CurStr)+1);
+        CharToWide(CurStr,&WideStr[0],WideStr.Size());
+#ifdef _WIN_ALL
+        if (ExpandEnvStr && *CurStr=='%')
+        {
+          // Expanding environment variables in Windows version.
+          wchar ExpName[NM];
+          DWORD Result=ExpandEnvironmentStringsW(&WideStr[0],ExpName,ASIZE(ExpName));
+          Expanded=Result!=0 && Result<ASIZE(ExpName);
+          if (Expanded)
+            List->AddString(ExpName);
+        }
+#endif
+        if (!Expanded)
+          List->AddString(&WideStr[0]);
       }
       CurStr=NextStr+1;
       while (*CurStr=='\r' || *CurStr=='\n')
         CurStr++;
     }
   }
-  return(true);
+  return true;
 }
 
 
 bool IsUnicode(byte *Data,int Size)
 {
   if (Size<4 || Data[0]!=0xff || Data[1]!=0xfe)
-    return(false);
+    return false;
   for (int I=2;I<Size;I++)
     if (Data[I]<32 && Data[I]!='\r' && Data[I]!='\n')
-      return(true);
-  return(false);
+      return true;
+  return false;
 }
