@@ -309,7 +309,6 @@
 #define   MOwnersBroken      "\nERROR: %s group and owner data are corrupt\n"
 #define   MSetOwnersError    "\nWARNING: Cannot set %s owner and group\n"
 #define   MErrLnkRead        "\nWARNING: Cannot read symbolic link %s"
-#define   MErrCreateLnk      "\nWARNING: Cannot create link %s"
 #define   MSymLinkExists     "\nWARNING: Symbolic link %s already exists"
 #define   MAskRetryCreate    "\nCannot create %s. Retry ?"
 #define   MListMACHead1      "\n Mac OS file type:  %c%c%c%c  ; "
@@ -382,3 +381,6 @@
 #define   MCopyError         "\nCannot copy %s to %s."
 #define   MCopyErrorHint     "\nYou need to unpack the entire archive to create file copy entries."
 #define   MCopyingData       "\nCopying data"
+#define   MErrCreateLnkS     "\nCannot create symbolic link %s"
+#define   MErrCreateLnkH     "\nCannot create hard link %s"
+#define   MNeedAdmin         "\nYou may need to run RAR as administrator"
